/**
 * Represents a service in the military
 */
package com.kvnallsn.militarynews;

import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import android.graphics.drawable.Drawable;

/**
 * @author Kevin Allison
 *
 */
public class ListImageItem {
	
	private String name = null;
	private String subText = null;
	private Drawable logo_d = null;
	public boolean loadingImage = true;
	public URL imageURL = null;
	public String link = null;
	
	/*
	 * Constructor - Creates a Military object
	 * 
	 * @param name The title of the article
	 * @param logo The image to draw (optional)
	 * @param subText The subtext to add (optional)
	 */
	public ListImageItem(String name, Drawable logo, String subText) {
		this.name = name;
		this.logo_d = logo;
		this.subText = subText;
	}
	
	/*
	 * For use when the URL is in string form
	 */
	public ListImageItem(String name, final String logoURLString, String subText) {
		this.name = name;
		try {
			if (!logoURLString.equals("")) {
				imageURL = new URL(logoURLString);
			} 
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} finally {
			if (logo_d == null) {
				logo_d = ServiceInfo.EMPTY_DRAWABLE;
			}
		}
		this.subText = subText;
	}
	
	public ListImageItem(String name, URL logoURL, String subText) {
		this.name = name;
		try {
			InputStream is = (InputStream) logoURL.getContent();
			logo_d = Drawable.createFromStream(is, "src");
//			/logo_d = ServiceInfo.EMPTY_DRAWABLE;
		} catch (Exception e) {
			e.printStackTrace();
		}
		this.subText = subText;
	}
	
	public ListImageItem() { }
	
	/*	
	 * Get the service branches name
	 */
	public String getName() {
		return name;
	}
	
	/*
	 * Get the subText passed in
	 */
	public String getSubText() {
		if (this.subText == null && ServiceInfo.tabsActive) {
			this.subText = "No Date Available";
		}
		return this.subText;
	}
	
	public Drawable getLogoDrawable() {
		return logo_d;
	}
}
