/**
 * Adapter For Military Use
 */
package com.kvnallsn.militarynews;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

/**
 * @author Kevin Allison
 *
 */
public class ListImageItemAdapter extends BaseAdapter {

	protected Context context;
	protected List<ListImageItem> serviceList;
	//private static final int ICON_ID = 0;
	//private static final int TITLE_ID = 1;
	//private static final int DESC_ID = 2;
	
	/*
	 * Constructor
	 */
	public ListImageItemAdapter(Context ctx, List<ListImageItem> serviceList) {
		this.context = ctx;
		this.serviceList = serviceList;
	}
	
	public int getCount() {
		return serviceList.size();
	}

	public Object getItem(int position) {
		return serviceList.get(position);
	}

	public long getItemId(int position) {
		return position;
	}
	
	public void addItem(ListImageItem item) {
		serviceList.add(item);
	}

	public View getView(int position, View v, ViewGroup parent) {
		ListImageItem item = serviceList.get(position);
		
		return new ListImageItemAdapterView(this.context, item);
	}
}


class ListImageItemAdapterView extends LinearLayout {
	
	private ImageView iconControl;
	private ProgressBar progressBar;
	//private Handler mHandler = new Handler(); 
	
	public ListImageItemAdapterView(Context ctx, final ListImageItem service) {
		super(ctx);
		
		this.setOrientation(HORIZONTAL);
		
		LinearLayout.LayoutParams iconParams = new LinearLayout.LayoutParams(72, 72);
		iconParams.setMargins(10, 1, 10, 1);	
		iconParams.gravity = android.view.Gravity.CENTER_VERTICAL;
		iconControl = new ImageView(ctx);
		iconControl.setVisibility(GONE);
		
		if (service.getLogoDrawable() != null) {
			
			addView(iconControl, iconParams);
			if (service.imageURL != null) {
				progressBar = new ProgressBar(ctx);
				LinearLayout.LayoutParams progressBarParams = new LinearLayout.LayoutParams(72, 72);
				progressBarParams.gravity = android.view.Gravity.CENTER_VERTICAL;
				progressBar.setIndeterminate(true);
				addView(progressBar, progressBarParams);
				iconControl.setVisibility(GONE);
				
				try {
					InputStream is = (InputStream) service.imageURL.getContent();
					final Drawable icon = Drawable.createFromStream(is, "src");
					iconControl.setImageDrawable(icon);
					progressBar.setVisibility(GONE);
					iconControl.setVisibility(VISIBLE);

				} catch (IOException e) {
					e.printStackTrace();
				}

			} else {
				iconControl.setVisibility(VISIBLE);
				iconControl.setImageDrawable(service.getLogoDrawable());
			}			
		}
		
		LinearLayout.LayoutParams textParams = new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		TextView textControl = new TextView(ctx);
		textParams.setMargins(5, 5, 1, 5);
		
		textControl.setTextSize(18f);			
		textControl.setText(service.getName());		
		textControl.setTextColor(Color.WHITE);	
		

		LinearLayout sideLayout = new LinearLayout(ctx);
		LinearLayout.LayoutParams linearParams = new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		sideLayout.setOrientation(LinearLayout.VERTICAL);

		TextView subText = new TextView(ctx);
		LinearLayout.LayoutParams subTextParams = new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		subTextParams.setMargins(5, 5, 1, 5);
		
		subText.setTextSize(14f);
		subText.setTextColor(Color.GRAY);
		subText.setText(service.getSubText());
		
		sideLayout.addView(textControl, textParams);
		sideLayout.addView(subText,subTextParams );
		addView(sideLayout, linearParams);	
	}
}
