/**
 * Show the main menu
 */
package com.kvnallsn.militarynews;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * @author Kevin Allison
 *
 */
public class MainMenu extends Activity {
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	getWindow().requestFeature(android.view.Window.FEATURE_NO_TITLE);
    	try {
	    	super.onCreate(savedInstanceState);
	    	ScrollView scroll = new ScrollView(this);
	    	scroll.setFillViewport(true);
	    	LinearLayout top = new LinearLayout(this);
	    	top.setOrientation(LinearLayout.VERTICAL);

	    	LayoutInflater inflater = getLayoutInflater();
	    	// Top Header (Banner)
	    	top.addView(inflater.inflate(R.layout.main_menu_header, null));
	    	
	    	//Service Feeds and Seperators
	    	top.addView(new MenuHeader(this, "Service Feeds"));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.army_logo, "Army", ServiceInfo.ARMY));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.navy_logo, "Navy", ServiceInfo.NAVY));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.af_logo, "Air Force", ServiceInfo.AIRFORCE));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.marines_logo, "Marine Corps", ServiceInfo.MARINES));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.coast_guard_logo, "Coast Guard", ServiceInfo.COASTGUARD));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.dod, "Department of Defense", ServiceInfo.DOD));
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	
	    	//Specialized Menu Items
	    	top.addView(new MenuHeader(this, "Options"));	
	    	top.addView(inflater.inflate(R.layout.seperator, null));
	    	top.addView(new MenuItem(this, R.drawable.save, "View Saved Items", ServiceInfo.SAVEDITEMS));
	    	
	    	scroll.addView(top);
	    	setContentView(scroll);
	    	
    	} catch(Exception e) {
    		Toast.makeText(this, e.toString(), Toast.LENGTH_LONG).show();
    	}
    }
	
	@Override
	public void onDestroy() {
		new CacheManagement().execute(this);
		super.onDestroy();
	}

	private class MenuItem extends LinearLayout {

		public MenuItem(Context ctx, int icon, CharSequence text, int id) {
			super(ctx);
			this.setOrientation(HORIZONTAL);
			
			ServiceInfo app = (ServiceInfo) MainMenu.this.getApplication();
			int imagePixels = app.getImageSize(48.0f);
			
			ImageView image = new ImageView(ctx);
			LayoutParams imageParams = new LayoutParams(imagePixels, imagePixels);
			imageParams.setMargins(10, 1, 5, 1);
			image.setImageResource(icon);
			image.setScaleType(android.widget.ImageView.ScaleType.FIT_XY);
			
			addView(image, imageParams);
			
			TextView textView = new TextView(ctx);
			LayoutParams textParams = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.FILL_PARENT);
			textParams.setMargins(10, 5, 1, 5);
			textView.setText(text);	
			textView.setGravity(android.view.Gravity.CENTER_VERTICAL);
			textView.setTextColor(Color.WHITE);
			textView.setTextSize(20f);
		
			addView(textView, textParams);
			this.setBackgroundDrawable(this.getContext().getResources().getDrawable(R.drawable.main_menu_item_selector));
			this.setClickable(true);
			this.setId(id);
			this.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {
					ServiceInfo.serviceCode = v.getId();
					startActivity(new Intent(MainMenu.this, NewsActivity.class));
				}
			});
		}
	}
	
	/*
	 * Header View
	 */
	private class MenuHeader extends LinearLayout {
		public MenuHeader(Context ctx, CharSequence title) {
			super(ctx);
			this.setBackgroundDrawable(ctx.getResources().getDrawable(R.drawable.main_menu_gradient));
			TextView text = new TextView(ctx);
			text.setText(title);
			text.setTextColor(Color.BLACK);
			text.setTextSize(18f);
			text.setPadding(5,0,0,0);
			
			addView(text);
		}
	}
}
