/*
 *  File: DisplayCreator.java
 *  
 *  $Id$
 *  
 *  Revisions:
 *       $Log$
 *       
 */
package com.kvnallsn.militarynews;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.xml.sax.InputSource;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

/*
 * Creates the proper display headers based on selected
 * military service
 */
public class RSSReader extends Activity implements OnItemClickListener {
	
	// Menu Constants	
	public static final int REFRESH = 0;
	public static final int SET_FAV = 1;
	public static final int NEWS_SOURCE = 2;
	
	// Context Menu Constants
	public static final int READ_MORE = 0;
	public static final int SAVE = 1;
	
	protected ListView lView;
	protected RSSFeed feed;
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle instance) {
		super.onCreate(instance);
		
		lView = new ListView(this);
		lView.setDivider(getResources().getDrawable(R.drawable.black_white_gradient));
		lView.setDividerHeight(2);
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateContextMenu(android.view.ContextMenu, android.view.View, android.view.ContextMenu.ContextMenuInfo)
	 */
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo info) {
		super.onCreateContextMenu(menu, v, info);
		menu.add(0, SAVE, 0, "Save Item");
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	public boolean onCreateOptionsMenu(Menu menu) {
	    menu.add(0, REFRESH, 0, "Refresh").setIcon(R.drawable.refresh);
	    menu.add(0, SET_FAV, 0, "Set Default Tab").setIcon(R.drawable.tab);
	    return true;
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	    case REFRESH:
	    	new FetchFeed().execute(ServiceInfo.getURL(ServiceInfo.READ_NEWS), ServiceInfo.getServiceName() + "News.tmp", "NoSkip");
	    	break;
	    case SET_FAV: // set Default Tab
	    	showDialog(SET_FAV);
	    	break;
	    case NEWS_SOURCE: // Select News Feed
	    	showDialog(NEWS_SOURCE);
	    	break;
	    }
	    return false;
	} 
	
	@Override 
	/*
	 * Helper method for backwards compatibility
	 */
	protected Dialog onCreateDialog(int id) {
		return onCreateDialog(id, null);
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateDialog(int, android.os.Bundle)
	 * 
	 * Create a dialog based on input
	 */
	protected Dialog onCreateDialog(int id, Bundle b) {
		if (id == SET_FAV) { 
			System.out.println("Setting Tab");
			final CharSequence[] items;
			if (ServiceInfo.serviceCode == ServiceInfo.MARINES) {
				CharSequence[] tempItems = { "News", "Twitter", "Youtube" };
				items = tempItems;;
			} else {
				CharSequence[] tempItems = { "News", "Twitter", "Blog", "Youtube" };
				items = tempItems;
			}
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setTitle("Select a tab");
			builder.setSingleChoiceItems(items, -1, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int position) {
					// Write the users preferences
  			        SharedPreferences settings = getSharedPreferences(ServiceInfo.getPrefsName(), 0);
			        SharedPreferences.Editor editor = settings.edit();
 			        editor.putInt("favTab", position);
 
			        // Commit the edits!
			        editor.commit();
				    dialog.dismiss();
				}
			});
			AlertDialog alert = builder.create();
			return alert;
		} else if (id == NEWS_SOURCE) {
			CharSequence[] tempItems = new CharSequence[3];
			final CharSequence[] items;
			switch (ServiceInfo.serviceCode) {
			case ServiceInfo.ARMY:
			case ServiceInfo.NAVY:
			case ServiceInfo.AIRFORCE:
			case ServiceInfo.MARINES:
			case ServiceInfo.DOD:
				tempItems[0] = "Google News";
				tempItems[1] = "Official News (.mil)";
				tempItems[2] = ServiceInfo.getServiceName() + "Times.com";
				break;
			case ServiceInfo.COASTGUARD:
				tempItems = new CharSequence[2];
				tempItems[0] = "Google News";
				tempItems[1] = "Official News (.mil)";
				break;
			}
			
			items = tempItems;
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setTitle("Select A News Source");
			builder.setSingleChoiceItems(items, -1, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int position) {
					// Write the users preferences
					SharedPreferences settings = getSharedPreferences(ServiceInfo.getPrefsName(), 0);
					SharedPreferences.Editor editor = settings.edit();
					editor.putInt("newsSource", position);
					
					new FetchFeed().execute(ServiceInfo.getNewsSource(position), ServiceInfo.getServiceName() + "News.tmp", "NoSkip");
					
					// Commit to save the changes
					editor.commit();
					dialog.dismiss();
				}
			});
			AlertDialog alert = builder.create();
			return alert;
		}
		
		return null; 
	}
	
	protected void setLoadingLayout() {
		final int PROGRESS = 20;
		RelativeLayout layout = new RelativeLayout(RSSReader.this);
		ProgressBar progress = new ProgressBar(RSSReader.this);
		progress.setId(PROGRESS);
		
		ServiceInfo app = (ServiceInfo) getApplication();
		int imagePixels = app.getImageSize(48.0f);
		RelativeLayout.LayoutParams progressParams = new RelativeLayout.LayoutParams(imagePixels, imagePixels);
		
		progressParams.addRule(RelativeLayout.CENTER_IN_PARENT);
		progress.setIndeterminate(true);
		layout.addView(progress, progressParams);
		
		TextView text = new TextView(this);
		RelativeLayout.LayoutParams textParams = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT, 
																				RelativeLayout.LayoutParams.WRAP_CONTENT);
		textParams.addRule(RelativeLayout.BELOW, PROGRESS);
		textParams.addRule(RelativeLayout.CENTER_IN_PARENT);
		textParams.setMargins(0,3,0,0);
		text.setText("Loading...");
		layout.addView(text,textParams);
		
		setContentView(layout);
	}
	
	/*
	 * Get the news from the RSS feed
	 * Feed is actually store XML file
	 */
	protected class FetchFeed extends AsyncTask<String, Void, ListImageItemAdapter> {
		
		private boolean errorOccured = false;
		
		protected void onPreExecute() {
			setLoadingLayout();
		}
		
		/*
		 * Retrieve the RSS feed from the given location and write it out as a XML text file
		 * 
		 * @param location The String Array passed in from doInBackground(...)
		 * 
		 * @return File The file for the RSS Parser to parse from
		 */
		private File fetchNews(String... location) {
			
			if ("NA".equals(location[0])) {
				return null;
			} 
			
			File tempFile = new File(getCacheDir(), location[1]);
			
			if(tempFile.exists() && "Skip".equals(location[2])) {
				return tempFile;
			}
			
			try {
				System.out.println("Fetching new content");
				// Set up the Uniform Resource Locator
				URL url = new URL(location[0]);
				
				HttpClient httpClient = new DefaultHttpClient();
				HttpResponse response = httpClient.execute(new HttpGet(url.toURI()));
				
				InputSource is = new InputSource(response.getEntity().getContent());
				
				if (tempFile.exists()) {
					tempFile.delete();
				}
				
				if (!tempFile.createNewFile()) { 
					System.out.println("File Not Created");
					return null;					
				}
				
				FileOutputStream fOut = new FileOutputStream(tempFile);
				OutputStreamWriter osw = new OutputStreamWriter(fOut);
				
				
				char[] buffer = new char[1024];
				try {
					Reader reader = new BufferedReader(new InputStreamReader(is.getByteStream()));
					int pos;
					while ((pos = reader.read(buffer)) != -1) {
						osw.write(buffer,0,pos);
					}
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					osw.close();
					fOut.close();
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			return tempFile;
		}
		
		/*
		 * Open the given file and parse it as a RSS feed
		 * 
		 * @param file The file containing the XML content to parse
		 * 
		 * @return RSSFeed The RSS feed broken down into the helper class RSSFeed.
		 * 				   Used to generate the List Adapter
		 */
		private RSSFeed parseFeed(File file) throws Exception {
			
			XmlPullParserFactory pullMaker;
			
			if (file != null) {
				try {
	
					pullMaker = XmlPullParserFactory.newInstance();
					XmlPullParser parser = pullMaker.newPullParser();
					
					parser.setInput(new java.io.FileReader(file));
					
					//State flags
					boolean inItem = false;
					boolean inTitle = false;
					boolean inURL = false;
					boolean inDate = false;
					RSSFeed rssFeed = new RSSFeed();
					RSSItem currentItem = null;
					
					int eventType = parser.getEventType();
					while (eventType != XmlPullParser.END_DOCUMENT) {
						switch (eventType) {
						case XmlPullParser.START_DOCUMENT:
							
							break;
						case XmlPullParser.START_TAG:
							if (parser.getName().compareTo("item") == 0) {
								currentItem = new RSSItem();
								inItem = true;
							} else if (parser.getName().compareTo("title") == 0) {
								if (inItem) {
									inTitle = true;
								}
							} else if (parser.getName().compareTo("link") == 0 && inItem) {
								inURL = true;
							} else if (parser.getName().compareTo("pubDate") == 0 && inItem) {
								inDate = true;
							}
							break;
						case XmlPullParser.END_TAG:
							if (parser.getName().equals("item")) {
								inItem = false;
								rssFeed.addItem(currentItem);
								currentItem = null;
							} else if (parser.getName().equals("title")) {
								inTitle = false;
							} else if (parser.getName().equals("link")) {
								inURL = false;
							} else if (parser.getName().equals("pubDate")) {
								inDate = false;
							}
							break;
						case XmlPullParser.TEXT:
							if (inItem) {
								if (inTitle) {
									currentItem.setTitle(parser.getText());
								} else if (inURL) {
									currentItem.setLink(parser.getText());
								} else if (inDate) {
									currentItem.setPubDate(parser.getText());
								}
							}
							break;
						}
						eventType = parser.next();
					}
					
					// Return the newly created RSS Feed
					return rssFeed;
				} catch (XmlPullParserException e) {
					RSSFeed errorFeed = new RSSFeed();
					RSSItem errorItem = new RSSItem();
					errorItem.setTitle("Oh No!  The RSS Feed is not well-formed!  Cannot read the feed!  Use another News Source until the author of this feed fixes the format.");
					errorFeed.addItem(errorItem);
					errorOccured = true;
					return errorFeed;
				} catch (Exception e) {
					e.printStackTrace();
					return null;
				} 
			} else {
				RSSFeed errorFeed = new RSSFeed();
				RSSItem errorItem = new RSSItem();
				errorItem.setTitle("Oh No!  The RSS Feed is not well-formed!  Cannot read the feed!  Use another News Source until the author of this feed fixes the format.");
				errorFeed.addItem(errorItem);
				errorOccured = true;
				return errorFeed;
			}
		}
		
		/*
		 * What to execute on the background thread
		 * 
		 * @param params The String array of required content.
		 * 			[0]: The URL location of the RSS Feed
		 * 			[1]: The Name of the File it is cached in - Generally in ServiceNameFeedtype.tmp.  e.g. AirForceNews.tmp vs AirForceTwitter.tmp
		 *  		[2]: 'Skip' or 'NoSkip' - Whether or not to skip over retrieving updated feeds from the servers
		 *  
		 *  @return ListImageItemAdapter The listview adapter used to show the content
		 */
		protected ListImageItemAdapter doInBackground(String... params) {
			
			try {
				feed = parseFeed(fetchNews(params));
				
				List<ListImageItem> imageList = new LinkedList<ListImageItem>();
				
				Iterator<RSSItem> iter = feed.getAllItems().iterator();
				
				for (;iter.hasNext();) {
					RSSItem rss = iter.next();
					imageList.add(new ListImageItem(rss.getTitle(), ServiceInfo.NULL_DRAWABLE, rss.getPubDate()));
				}
				
				ListImageItemAdapter adapter = new ListImageItemAdapter(RSSReader.this, imageList);
				
				return adapter;
				
			} catch (Exception e) {
				e.printStackTrace();
				List<ListImageItem> errorList = new LinkedList<ListImageItem>();
				errorList.add(new ListImageItem("Error processing data.  Please check your internet connection and try again.", ServiceInfo.NULL_DRAWABLE, null));
				ListImageItemAdapter errorAdapter = new ListImageItemAdapter(RSSReader.this, errorList);
				errorOccured = true;
				return errorAdapter;	
			} 
		}
		
		/*
		 * (non-Javadoc)
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 * 
		 * Executes after the background thread has completed.
		 * 
		 * @param adapter  The ListImageItemAdapter generated from the background proccess.
		 * 
		 */
		protected void onPostExecute(ListImageItemAdapter adapter) {
			if (adapter != null) {
				lView.setAdapter(adapter);
				if (!errorOccured)
					lView.setOnItemClickListener(RSSReader.this);
				lView.setSelection(0);
			}			
			
			/*if (ServiceInfo.newsOption == ServiceInfo.READ_MORE) {
				LinearLayout layout = new LinearLayout(RSSReader.this);
				layout.setOrientation(LinearLayout.VERTICAL);
				TextView text = new TextView(RSSReader.this);
				text.setText("Related Articles");
				text.setGravity(android.view.Gravity.CENTER_HORIZONTAL);
				text.setTextSize(24);
				
				layout.addView(text);
				layout.addView(lView);
				
				setContentView(layout);
			} else {*/
				setContentView(lView);
			//}
			//dismissDialog(PROGRESS_DIALOG);			
		}
	}

	/*
	 * (non-Javadoc)
	 * @see android.widget.AdapterView.OnItemClickListener#onItemClick(android.widget.AdapterView, android.view.View, int, long)
	 * 
	 * Executes when an item has been clicked
	 */
	public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
		Intent itemIntent = new Intent(this, ViewOnline.class);
		
		itemIntent.putExtra("link", feed.getItem(position)._link);
		
		startActivity(itemIntent);	
	}
} 

