/**
 * View more details on a RSS feed
 */
package com.kvnallsn.militarynews;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ExpandableListAdapter;
import android.widget.ExpandableListView;
import android.widget.ImageView;
import android.widget.SimpleExpandableListAdapter;
import android.widget.TextView;
import android.widget.Toast;

/**
 * @author Kevin Allison
 *
 */
public class ReadTwitter extends Activity {
	
	@Override
	public void onCreate(Bundle icicle) {
		getWindow().requestFeature(android.view.Window.FEATURE_NO_TITLE);
		super.onCreate(icicle);
		setContentView(R.layout.tweets);
		
		Intent startingIntent = getIntent();
		String content = "";
		String title = "";
		String imageURL = "";
		String link = "";
		boolean retweet = false;
		String[] tags = null;
		
		if (startingIntent != null) {
				content = startingIntent.getStringExtra("content");
				title = startingIntent.getStringExtra("title");
				imageURL = startingIntent.getStringExtra("ImageURI");
				retweet = startingIntent.getBooleanExtra("retweet", false);
				tags = startingIntent.getStringArrayExtra("tags");
				link = startingIntent.getStringExtra("link");	
				System.out.println(link);
		} else {
			Toast.makeText(this, "Error Retreiving Tweet Info", Toast.LENGTH_SHORT).show();
		}
		
		TextView titleText = (TextView) findViewById(R.id.DescriptionTitle);
		titleText.setText(title);
		
		ImageView picture = (ImageView) findViewById(R.id.DescriptionPic);
		
		try {
			URL url = new URL(imageURL);
			HttpClient httpClient = new DefaultHttpClient();
			//HttpResponse response = httpClient.execute(new HttpGet(url.toURI()));
			
			InputStream is = (InputStream) httpClient.execute(new HttpGet(url.toURI())).getEntity().getContent();
			Drawable d = Drawable.createFromStream(is, "twitterpic");
			picture.setImageDrawable(d);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		TextView retweetText = (TextView) findViewById(R.id.DescriptionRetweet);
		if (retweet) {	
			retweetText.setText(" -- Retweet");
		} 
		
		TextView contentText = (TextView) findViewById(R.id.DescriptionText);
		contentText.setText(content);
		
		// Create the expandable list view for the tags
		List<Map<String, String>> parentData = new ArrayList<Map<String, String>>();
		List<List<Map<String, String>>> childData = new ArrayList<List<Map<String, String>>>();
		
		Map<String, String> curGroupMap = new HashMap<String,String>();
		curGroupMap.put("TAGS", "Tags");
		parentData.add(curGroupMap);
		
		List<Map<String, String>> children = new ArrayList<Map<String, String>>();
		for (String s : tags) {
			 Map<String, String> curChildMap = new HashMap<String, String>();
			 curChildMap.put("TAGS", s);
			 children.add(curChildMap);
		}
		
		if (children.size() == 0) {
			 Map<String, String> curChildMap = new HashMap<String, String>();
			 curChildMap.put("TAGS", "None");
			 children.add(curChildMap);
		}
		
		childData.add(children);
		
		ExpandableListAdapter mAdapter = new SimpleExpandableListAdapter(this, parentData, android.R.layout.simple_expandable_list_item_1, 
												new String[] {"TAGS"}, new int[] { android.R.id.text1 }, childData, android.R.layout.simple_expandable_list_item_1,
												new String[] {"TAGS"}, new int[] { android.R.id.text1 });
		
		
		ExpandableListView list = (ExpandableListView) findViewById(R.id.DescriptionListView);
		list.setAdapter(mAdapter); 
			
		// End expandable list view creation
		
		
		// Determine whether or not to make the button active
		Button followLink = (Button) findViewById(R.id.DescriptionButton);
		if ("".equals(link)) {
			followLink.setVisibility(Button.GONE);
		} else {
			final String linkFinal = link;
			followLink.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {
					Intent itemIntent = new Intent(ReadTwitter.this, ViewOnline.class);
					
					itemIntent.putExtra("link", linkFinal);
					
					startActivity(itemIntent);	
				}
			});
		}
	}

}
