/**
 * Read the saved items
 */
package com.kvnallsn.militarynews;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Toast;

/**
 * @author Kevin Allison
 *
 */
public class SavedReader extends Activity {

	private List<SavedItem> list;
	public static final int DELETE = 0;
	private ListView listView;
	private LinearLayout top;
	private int type;
	
	public void onCreate(Bundle icicle) {
		getWindow().requestFeature(android.view.Window.FEATURE_NO_TITLE);
		super.onCreate(icicle);
		
		top = new LinearLayout(this);
		type = getIntent().getIntExtra("type", ServiceInfo.READ_NEWS);
		list = new ArrayList<SavedItem>();
		listView = new ListView(this);
		listView.setId(android.R.id.list);
		
		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
				if (type == ServiceInfo.READ_NEWS || type == ServiceInfo.BLOG) {
					startActivity(new Intent().setClass(SavedReader.this, ViewOnline.class).putExtra("link", list.get(position).link));				
				} else if (type == ServiceInfo.TWITTER) {
					TweetInfo info = new TweetInfo(list.get(position).title, list.get(position).link, list.get(position).imageURL);
					new ParseText(SavedReader.this, info).execute();
				} else if (type == ServiceInfo.YOUTUBE) {
					startActivity(new Intent(Intent.ACTION_VIEW, android.net.Uri.parse(list.get(position).link)));
				}
			}
		});
		
		registerForContextMenu(listView);
		new refreshListView().execute();
		top.addView(listView);
		setContentView(top);
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	public boolean onCreateOptionsMenu(Menu menu) {
	    return true;
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
		//ServiceInfo app = (ServiceInfo)SavedReader.this.getApplication();
	    switch (item.getItemId()) {
	    }
	    return false;
	} 
	
	/*
	 * Handles refreshing the listView in the event and item is deleted and on creation
	 */
	private class refreshListView extends AsyncTask<Void, Void, ListImageItemAdapter> {
		
		protected void onPreExecute() {
		}
		
		@Override
		protected ListImageItemAdapter doInBackground(Void... params) {
			ServiceInfo app = (ServiceInfo)SavedReader.this.getApplication();
			boolean fromURL = false;
			String table = "";
			switch (type) {
			case ServiceInfo.READ_NEWS:
				table = DatabaseLayer.NEWS_TABLE;
				break;
			case ServiceInfo.TWITTER:
				table = DatabaseLayer.TWITTER_TABLE;
				fromURL = true;
				break;
			case ServiceInfo.BLOG:
				table = DatabaseLayer.BLOG_TABLE;
				break;
			case ServiceInfo.YOUTUBE:
				table = DatabaseLayer.VIDEO_TABLE;
				fromURL = true;
				break;
			}
			
			// Get the items
			list = app.dbLayer.fetchAll(table, type);
			
			// Generate the list items
			List<ListImageItem> itemList = new ArrayList<ListImageItem>();
			
			for (SavedItem item : list) {
				Drawable image;
				if (fromURL) {
					itemList.add(new ListImageItem(item.title, item.imageURL, item.date));
				} else {
					if (item.imageURL.equals("ArmyPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.army_logo);
					} else if (item.imageURL.equals("NavyPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.navy_logo);
					} else if (item.imageURL.equals("AFPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.af_logo);
					} else if (item.imageURL.equals("MarinesPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.marines_logo);
					} else if (item.imageURL.equals("CGPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.coast_guard_logo);
					} else if (item.imageURL.equals("DoDPrefs")) {
						image = SavedReader.this.getResources().getDrawable(R.drawable.dod);
					} else {
						image = ServiceInfo.NULL_DRAWABLE;	
					} 
					itemList.add(new ListImageItem(item.title, image, item.date));
				}
			}			
			return new ListImageItemAdapter(SavedReader.this, itemList);
		}		
		
		protected void onPostExecute(ListImageItemAdapter adapter) {
			listView.setAdapter(adapter);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateContextMenu(android.view.ContextMenu, android.view.View, android.view.ContextMenu.ContextMenuInfo)
	 */
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo info) {
		super.onCreateContextMenu(menu, v, info);
		menu.add(0, DELETE, 0, "Delete Item");
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onContextItemSelected(android.view.MenuItem)
	 */
	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {
		case DELETE:
			ServiceInfo app = (ServiceInfo) getApplication();
			String table;
			switch (type) {
			case ServiceInfo.READ_NEWS:
				table = DatabaseLayer.NEWS_TABLE;
				break;
			case ServiceInfo.TWITTER:
				table = DatabaseLayer.TWITTER_TABLE;
				break;
			case ServiceInfo.BLOG:
				table = DatabaseLayer.BLOG_TABLE;
				break;
			case ServiceInfo.YOUTUBE:
				table = DatabaseLayer.VIDEO_TABLE;
				break;
			default:
				return false;
			}
			
			app.dbLayer.open();
			if (app.dbLayer.deleteItem(table, list.get(info.position).id)) {
				Toast.makeText(this, "Item successfully deleted.", Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(this, "Error deleting item.", Toast.LENGTH_SHORT).show();	
			}
			new refreshListView().execute();
			return true;
		}
		return super.onContextItemSelected(item);
	}
}

/*
 * Class to represent the items stored in the list
 */
class SavedItem {
	public String title;
	public String link;
	public String linkMore;
	public String imageURL;
	public String date;
	public long id;
	
	public SavedItem(String title, String date, String link, String linkMore, long id, String imageURL) {
		this.title = title;
		this.date = date;
		this.link = link;
		this.linkMore = linkMore;
		this.id = id;
		this.imageURL = imageURL;
	}
}
