/**
 * Static class used to represent the currently selected branch
 */
package com.kvnallsn.militarynews;

import android.app.Application;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;


/**
 * @author Kevin Allison
 *
 */
public class ServiceInfo extends Application {
	public static final Drawable NULL_DRAWABLE = null;
	public static final Drawable EMPTY_DRAWABLE = new BitmapDrawable();
	
	public static final int ARMY = 0;
	public static final int NAVY = 1;
	public static final int AIRFORCE = 2;
	public static final int MARINES = 3;
	public static final int COASTGUARD = 4;
	public static final int DOD = 5;
	public static final int JOINTCHIEFS = 6;
	public static final int SAVEDITEMS = 7;
	
	public static int serviceCode = -1;
	public static boolean tabsActive = false;
	
	public static final int READ_NEWS = 0;
	public static final int TWITTER = 1;
	public static final int BLOG = 2;
	public static final int YOUTUBE = 3;
	public static final int READ_MORE = 4;
	
	public static String[] ARMY_OPTIONS = { "",
											"USARMY", "http://armylive.dodlive.mil/index.php/feed/",
											"http://gdata.youtube.com/feeds/api/users/soldiersmediacenter/uploads?orderby=updated" };
	
	public static String[] NAVY_OPTIONS = { "", "USNavy",
											"http://navylive.dodlive.mil/index.php/feed/",
											"http://gdata.youtube.com/feeds/api/users/UnitedStatesNavy/uploads?orderby=updated" };
	
	public static String[] AIR_FORCE_OPTIONS = { "", "usairforce",
												 "http://airforcelive.dodlive.mil/index.php/feed/", 
												 "http://gdata.youtube.com/feeds/api/users/afbluetube/uploads?orderby=updated" };
	
	public static String[] MARINES_OPTIONS = { "",
											  "usmc", "", "http://gdata.youtube.com/feeds/api/users/marines/uploads?orderby=updated" };
	
	public static String[] COAST_GUARD_OPTIONS = { "", "USCG",
												   "http://coastguard.dodlive.mil/index.php/feed/",
												   "http://gdata.youtube.com/feeds/api/users/USCGImagery/uploads?orderby=updated" };
	
	public static String[] DOD_OPTIONS = { "", "deptofdefense",
										   "http://www.dodlive.mil/index.php/feed/", 
										   "http://gdata.youtube.com/feeds/api/users/thepentagonchannel/uploads?orderby=updated" };
	
	/*
	 * Get the URL to load (if not viewing the news tab)
	 * 
	 * @return String The URL/Username of the selected option
	 */
	public static String getURL(int newsOption) {
	
		switch (serviceCode) {
		case ARMY:
			return ARMY_OPTIONS[newsOption];
		case NAVY:
			return NAVY_OPTIONS[newsOption];
		case AIRFORCE:
			return AIR_FORCE_OPTIONS[newsOption];
		case MARINES:
			return MARINES_OPTIONS[newsOption];
		case COASTGUARD:
			return COAST_GUARD_OPTIONS[newsOption];
		case DOD:
			return DOD_OPTIONS[newsOption];
		}
		
		return "";
	}
	
	/*
	 * Get the correct news source based on the selected option
	 * 
	 * @param newsNum The number corresponding to the news source.  1 -- Google News,  2 -- Official News (.mil),  3 -- Military Times
	 * 
	 * @return String The URL of the RSS feed
	 */
	public static String getNewsSource(int newsNum) {
		String[] options = new String[3];
		switch (serviceCode) {
		case ARMY:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=us+army&cf=all&output=rss" ;
			options[1] = "http://www.army.mil/rss/feeds/frontpage.xml";
			options[2] = "http://www.armytimes.com/rss_news.php";
			break;
		case NAVY:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=us+navy&cf=all&output=rss";
			options[1] = "http://www.navy.mil/xml/tops.xml";
			options[2] = "http://www.navytimes.com/rss_news.php";
			break;
		case AIRFORCE:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=us+air+force&output=rss";
			options[1] = "http://www.af.mil/rss/TopStoriesByTab.asp?tabId=152";
			options[2] = "http://www.airforcetimes.com/rss_news.php";
			break;
		case MARINES:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=us+marine+corps&cf=all&output=rss";
			options[1] = "http://www.marines.mil/tnsw/_layouts/feed.aspx?xsl=1&web=%2Ftnsw&page=e5ffbf72-c965-402a-a695-7b4cfcc2715b&wp=19676c27-d033-4d82-b4b9-1f50498cee99";
			options[2] = "http://www.marinecorpstimes.com/rss_news.php";
			break;
		case COASTGUARD:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=us+coast+guard&cf=all&output=rss";
			options[1] = "http://www.piersystem.com/go/feed/786/ru/rss20/";
			options[2] = "";
			break;
		case DOD:
			options[0] = "http://news.google.com/news?pz=1&cf=all&ned=us&hl=en&q=department+of+defense&cf=all&output=rss";
			options[1] = "http://www.defense.gov/news/rss_releases2.xml";
			options[2] = "http://www.militarytimes.com/rss_news.php";
		}
		
		return options[newsNum];
	}
	
	public static String getPrefsName() {
		switch (serviceCode) {
		case ARMY:
			return "ArmyPrefs";
		case NAVY:
			return "NavyPrefs";
		case AIRFORCE:
			return "AFPrefs";
		case MARINES:
			return "MarinesPrefs";
		case COASTGUARD:
			return "CGPrefs";
		case DOD:
			return "DoDPrefs";
		}
		
		return "";
	}
	
	public static String getServiceName() {
		switch (serviceCode) {
		case ARMY:
			return "Army";
		case NAVY:
			return "Navy";
		case AIRFORCE:
			return "AirForce";
		case MARINES:
			return "MarineCorps";
		case COASTGUARD:
			return "Coast Guard";
		case DOD:
			return "Military";
		}
		
		return "";
	}
	
	public int getImageSize(float mdpi_size) {
		// Convert image size to pixels
		//final float IMAGE_SIZE = 48.0f;
		final float density = this.getResources().getDisplayMetrics().density;
		return (int) (mdpi_size * density + .5f);
	}
	
	// Store the database
	protected DatabaseLayer dbLayer;
	
	// Called on creation of Application
	public void onCreate() {
		super.onCreate();
		
		dbLayer = new DatabaseLayer(this);
		//dbLayer = dbLayer.open();
		//dbLayer.upgrade();
	}

	@Override
	public void onTerminate() {
		dbLayer.close();
		super.onTerminate();
	}
	
	public void changeDatabase(DatabaseLayer db) {
		this.dbLayer = db;
	}
}
