/**
 * Retrieves and displays Twitter feeds
 */
package com.kvnallsn.militarynews;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import winterwell.jtwitter.Twitter;
import winterwell.jtwitter.TwitterException;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Toast;

/**
 * @author Kevin Allison
 *
 */
public class TwitterReader extends RSSReader implements OnItemClickListener {
	
	private List<Twitter.Status> status;
	
	/*
	 * Creates a new TwitterReader object
	 * 
	 * @param ctx The context in which to create the Twitter reader
	 */
	public void onCreate(Bundle icicile) {
		super.onCreate(icicile);
				
		registerForContextMenu(lView);
		//ServiceInfo.newsOption = ServiceInfo.TWITTER;
		new GetNews().execute();
	}
	
	private void displayError(String text) {
		List<String> s = new LinkedList<String>();
		s.add(text);
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, s);
		lView.setAdapter(adapter);
	}
	
	private class GetNews extends AsyncTask<Void, Void, ListImageItemAdapter> {
		
		protected void onPreExecute() {
			setLoadingLayout();
		}
		
		@Override
		protected ListImageItemAdapter doInBackground(Void... params) {
			
			try {
				Twitter twitter = new Twitter();
			
				twitter.setMaxResults(20);
				status = twitter.getUserTimeline(ServiceInfo.getURL(ServiceInfo.TWITTER));
				
				List<ListImageItem> imageList = new LinkedList<ListImageItem>();
				
				Iterator<Twitter.Status> iter = status.iterator();
				
				for (;iter.hasNext();) {
					Twitter.Status s = iter.next();
					ListImageItem temp = new ListImageItem(s.getText(), ServiceInfo.NULL_DRAWABLE, s.getCreatedAt().toGMTString());
					imageList.add(temp);
				}
				
				ListImageItemAdapter adapter = new ListImageItemAdapter(TwitterReader.this, imageList);
				
				return adapter;
				
			} catch (TwitterException.E50X e) {
				displayError("Twitter is busy right now, please try again in a minute or so");
			} catch (TwitterException.RateLimit e) {
				displayError("Two many Twitter requests! Twitter Overload! Please try again in an hour.");
			} catch (TwitterException e) {
				displayError("Error retrieving tweets.  Please check your to ensure you are connected to the interent");
			} catch (RuntimeException e) {
				displayError("Twitter is busy right now, please try again in a minute or so");
			}
			
			return null;
		}
		
		protected void onPostExecute(ListImageItemAdapter adapter) {
			lView.setAdapter(adapter);
			lView.setOnItemClickListener(TwitterReader.this);
			lView.setSelection(0);
			
			setContentView(lView);
		}
		
	}

	public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
		try {
			if (status != null) {
				new ParseText(this, status.get(position)).execute();
			}
		} catch (Exception e) {
			System.out.println("Error: " + e.toString());
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onContextItemSelected(android.view.MenuItem)
	 */
	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {
		case SAVE:
			Twitter.Status tweet = status.get(info.position);
			ServiceInfo app = (ServiceInfo) TwitterReader.this.getApplication();
			app.dbLayer.open();
			if(app.dbLayer.addItem(DatabaseLayer.TWITTER_TABLE, tweet.text, tweet.createdAt.toLocaleString(),
									tweet.user.name, "", tweet.user.profileImageUrl.toString())) {
				Toast.makeText(this, "Tweet successfully saved.", Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(this, "Tweet is already saved.", Toast.LENGTH_SHORT).show();
			}
			app.dbLayer.close();
		}
		return super.onContextItemSelected(item);
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	    case 0:
	    	new GetNews().execute();
	    	break;
	    case 1: // set Default Tab
	    	showDialog(SET_FAV);
	    	break;
	    }
	    return false;
	} 
}

class ParseText extends AsyncTask<Void, Void, Intent> {

	private Context ctx;
	private TweetInfo info;
	
	public ParseText(Context ctx, TweetInfo info) {
		this.ctx = ctx;
		this.info = info;
	}
	
	public ParseText(Context ctx, Twitter.Status status) {
		this.ctx = ctx;
		this.info = new TweetInfo(status.text, status.user.name, status.user.profileImageUrl.toString());
	}

	protected Intent doInBackground(Void... params) {

		int startIndex = info.text.indexOf("http://");
		if (startIndex != -1) {
			int endIndex = info.text.indexOf(" ", startIndex);
			
			if (endIndex == -1) {
				info.link = info.text.substring(startIndex);
			} else {
				info.link = info.text.substring(startIndex, endIndex);
			}
			
			info.text = info.text.replace(info.link, "");
		}
		
		if ("RT".equals(info.text.substring(0, 2))) {
			info.retweet = true;
			info.text = info.text.substring(2); // Remove the RT
		}
		
		int charCount = info.text.replaceAll("[^#]", "").length();
		info.tags = new String[charCount];
		
		int i=0;
		startIndex = info.text.indexOf("#");
		while (i < charCount) { 
			int endIndex = info.text.indexOf(" ", startIndex);

			if (endIndex == -1) {
				info.tags[i] = info.text.substring(startIndex);
			} else {
				info.tags[i] = info.text.substring(startIndex, endIndex);
			}
			
			info.text = info.text.replace(info.tags[i], "");
			++i;
		}
		
		Intent itemIntent = new Intent(ctx, ReadTwitter.class);
		
		//System.out.println(info.profileImageUrl);
		
		itemIntent.putExtra("link", info.link);
		itemIntent.putExtra("content", info.text);
		itemIntent.putExtra("title", info.username);
		itemIntent.putExtra("retweet", info.retweet);
		itemIntent.putExtra("tags", info.tags);
		itemIntent.putExtra("ImageURI", info.profileImageUrl);

		return itemIntent;
	}		
	
	protected void onPostExecute(Intent intent) {
		ctx.startActivity(intent);
	}
}

class TweetInfo {
	public String link;
	public String text;
	public boolean retweet;
	public String[] tags;
	public String profileImageUrl;
	public String username;
	
	public TweetInfo(String text, String username, String imageUrl) {
		this.link = "";
		this.text = text;
		this.username = username;
		this.profileImageUrl = imageUrl;
	}
}
