/**
 * Reads Youtube generated XML files
 * Only overrides setContent in RSSReader
 */
package com.kvnallsn.militarynews;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.xml.sax.InputSource;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserFactory;

import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Toast;

/**
 * @author Kevin Allison
 *
 */
public class YoutubeReader extends RSSReader implements OnItemClickListener {
	
	public void onCreate(Bundle icicile) {
		super.onCreate(icicile);
				
		registerForContextMenu(lView);
		//ServiceInfo.newsOption = ServiceInfo.YOUTUBE;
		new GetNews().execute(true);
	}
	
	/*
	 * (non-Javadoc)
	 * @see android.app.Activity#onContextItemSelected(android.view.MenuItem)
	 */
	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {
		case SAVE:
			RSSItem newsItem = feed.getItem(info.position);
			ServiceInfo app = (ServiceInfo) YoutubeReader.this.getApplication();
			app.dbLayer.open();
			if (app.dbLayer.addItem(DatabaseLayer.VIDEO_TABLE, newsItem._title, newsItem._pubDate, newsItem._link, "", newsItem.getImageUrl().toString())) {
				Toast.makeText(this, "Video successfully saved.", Toast.LENGTH_SHORT).show();
			} else {
				Toast.makeText(this, "Video is already saved.", Toast.LENGTH_SHORT).show();
			}
			app.dbLayer.close();
			return true;
		}
		return super.onContextItemSelected(item);
	}
	
	private class GetNews extends AsyncTask<Boolean, Void, ListImageItemAdapter> {
		
		private ListImageItemAdapter adapter;
		private boolean errorOccured = false;
		
		protected void onPreExecute() {
			lView.setAdapter(adapter);
			setLoadingLayout();
		}
		
		/*
		 * Retrieve the RSS feed from the given location and write it out as a XML text file
		 * 
		 * @param location The String Array passed in from doInBackground(...)
		 * 
		 * @return File The file for the RSS Parser to parse from
		 */
		private File fetchNews(boolean skip) {
			
			File tempFile = new File(getCacheDir(), ServiceInfo.getServiceName() + "Youtube.tmp");
			
			if(tempFile.exists() && skip) {
				return tempFile;
			}
			
			try {
				// Set up the Uniform Resource Locator
				URL url = new URL(ServiceInfo.getURL(ServiceInfo.YOUTUBE));
				
				HttpClient httpClient = new DefaultHttpClient();
				HttpResponse response = httpClient.execute(new HttpGet(url.toURI()));
				
				InputSource is = new InputSource(response.getEntity().getContent());
				
				if (tempFile.exists()) {
					System.out.println("File Deleted");
					tempFile.delete();
				}
				
				if (!tempFile.createNewFile()) { 
					System.out.println("File Not Created");
					return null;					
				}
				
				FileOutputStream fOut = new FileOutputStream(tempFile);
				OutputStreamWriter osw = new OutputStreamWriter(fOut);
				
				
				char[] buffer = new char[1024];
				try {
					Reader reader = new BufferedReader(new InputStreamReader(is.getByteStream()));
					int pos;
					while ((pos = reader.read(buffer)) != -1) {
						osw.write(buffer,0,pos);
					}
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					osw.close();
					fOut.close();
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			return tempFile;
		}
		
		@Override
		protected ListImageItemAdapter doInBackground(Boolean... params) {
			
			XmlPullParserFactory pullMaker;
			try {
				
				pullMaker = XmlPullParserFactory.newInstance();
				XmlPullParser parser = pullMaker.newPullParser();
				
				parser.setInput(new FileReader(fetchNews(params[0])));
				
				//State flags
				boolean inItem = false;
				boolean inTitle = false;
				boolean inDate = false;
				RSSItem currentItem = null;
				//ListImageItem currentListItem = null;
				feed = new RSSFeed();
				
				int eventType = parser.getEventType();
				while (eventType != XmlPullParser.END_DOCUMENT) {
					switch (eventType) {
					case XmlPullParser.START_DOCUMENT:
						break;
					case XmlPullParser.START_TAG:
						if (parser.getName().compareTo("entry") == 0) {
							currentItem = new RSSItem();
							//currentListItem = new ListImageItem();
							inItem = true;
						} else if (parser.getName().compareTo("title") == 0) {
							if (inItem) {
								inTitle = true;
							}
						} else if (parser.getName().compareTo("link") == 0 && inItem) {
							String type = parser.getAttributeValue(null, "rel");
							if (type.equals("alternate")) {
								currentItem.setLink(parser.getAttributeValue(null, "href"));
							}
						} else if (parser.getName().compareTo("published") == 0 && inItem) {
							inDate = true;
						} else if (parser.getName().equals("media:thumbnail") && inItem) {
							if (parser.getAttributeValue(null, "height").equals("90")) {
								currentItem.setImageURL(parser.getAttributeValue(null, "url"));
							}
						}
						break;
					case XmlPullParser.END_TAG:
						if (parser.getName().equals("entry")) {
							inItem = false;
							if (currentItem == null)
								System.out.println("Oh Noes!!!");
							feed.addItem(currentItem);
							currentItem = null;
						} else if (parser.getName().equals("title")) {
							inTitle = false;
						} else if (parser.getName().equals("published")) {
							inDate = false;
						}
						break;
					case XmlPullParser.TEXT:
						if (inItem) {
							if (inTitle) {
								currentItem.setTitle(parser.getText());
							} else if (inDate) {
								currentItem.setPubDate(parser.getText());
							} 
						}
						break;
					}
					eventType = parser.next();
				} 
				
				List<ListImageItem> imageList = new LinkedList<ListImageItem>();
				
				Iterator<RSSItem> iter = feed.getAllItems().iterator();
				
				for (;iter.hasNext();) {
					RSSItem rss = iter.next();
					// rss.getTitle() String
					// rss.getImageUrl() URL
					// rss.getPubDate() String
					ListImageItem temp = new ListImageItem(rss.getTitle(), rss.getImageUrl(), rss.getPubDate());
					imageList.add(temp);
				}
				
				ListImageItemAdapter adapter = new ListImageItemAdapter(YoutubeReader.this, imageList);
				
				//errorOccured = true;
				return adapter;
				
			} catch (Exception e) {
				System.out.println(e.toString());
				e.printStackTrace();
				List<ListImageItem> errorList = new LinkedList<ListImageItem>();
				errorList.add(new ListImageItem("Error processing data.  Please check your internet connection and try again.", ServiceInfo.NULL_DRAWABLE, null));
				ListImageItemAdapter errorAdapter = new ListImageItemAdapter(YoutubeReader.this, errorList);
				errorOccured = true;
				return errorAdapter;	
			}
		}
		
		protected void onPostExecute(ListImageItemAdapter adapter) {
			lView.setAdapter(adapter);
			if (!errorOccured)
				lView.setOnItemClickListener(YoutubeReader.this);
			lView.setSelection(0);
			
			setContentView(lView);
		}
		
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	    case 0: //Refresh
	    	new GetNews().execute(false);
	    	break;
	    case 1: // set Default Tab
	    	showDialog(SET_FAV);
	    	break;
	    }
	    return false;
	}
	

	public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
		
		startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(feed.getItem(position)._link)));		
	}
}
